/*
 * MTS.h
 *
 *  Created on: Dec 17, 2010
 *      Author: sbak
 */

#ifndef MTS_H_
#define MTS_H_

#include "common.h"
#include "CovarianceMatrix.h"
#include "SignatureInterface.h"
#include "FeatureInterface.h"
#include "SignatureModel.h"
#include <qdatetime.h>
#include <qstring.h>
#include <typeinfo>

#define MTS_DEBUG_TIME
#define IMG_FORMAT "png"

enum WKERNEL {Uniform, Epanechnikov, Sigma, Cross, ReadFromFile};


class MTS :public SignatureInterface {

	std::string ID;
	std::string source_link;
	QDateTime start_t;			//time when signature has been created
	QDateTime end_t;			//time when signature has been updated for the last time
	IplImage* i_image;			//input image (the last image used for signature extraction)
	IplImage* p_image;			//scaled and normalized image (after processing)
	int nframes;

	std::vector<float> lastDistance;	//keeps distance values per feature computed in the last getDistance() call

	//MODEL INFO
	SignatureModel* model;
	std::vector<bool> _integrals;

	std::vector<FeatureInterface*> features;

	void rescale(std::vector<float>& values);
	std::vector<float> getWeigths();
	std::vector<float> kernelFromFile(std::string file);
	std::vector<float> getSigmaWeights();
	std::vector<float> getEpanechnikovWeights();
	std::vector<float> getCrossWeights();


	FeatureInterface* build(FeatureInterface* feature, CovarianceMatrix& engine);
	FeatureInterface* build(FeatureInterface* feature);
	bool isValid(FeatureInterface* feature, CovarianceMatrix& engine);
	FeatureInterface* getInstance(std::string);


public:

	MTS();
	~MTS();

	MTS(std::string src_link);
	MTS& operator=(const MTS& s);
	MTS(const MTS& m);
	float operator-(const MTS s);
	bool operator!=(const MTS s);



	void create(std::string ID, IplImage* img, lti::channel* mask = NULL, std::string config = "config.xml");
	void update(IplImage* _img, lti::channel* mask = NULL);
	float getDistance(MTS& sig);

	void releaseImages();

	//GUI functions
	std::string getID();
	void draw(IplImage* img, CvRect roi = cvRect(0,0,0,0));
	void drawROI(IplImage* img, CvRect roi = cvRect(0,0,0,0));
	IplImage* getInputImage();
	IplImage* getProcessedImage();
	IplImage* getDistanceImage();

	std::vector<float> getDistanceVector(MTS& sig);

	float getDistance(SignatureInterface* sig);


	std::vector<float> serialize();
	IplImage* getDisplayImage(){return getProcessedImage();};
	IplImage* getDebugImage(){return getDistanceImage();};
	void display(IplImage* img, CvRect roi){drawROI(img, roi);};
	std::string getSource(){return source_link;};
	void save(std::string fileName);
	bool load(std::string fileName);
	bool created();
};

#endif /* MTS_H_ */
