/*
 * SignatureModel.h
 *
 *  Created on: May 30, 2013
 *      Author: sbak
 */

#ifndef SIGNATUREMODEL_H_
#define SIGNATUREMODEL_H_

#include "FeatureInterface.h"

#include "qt_support.h"

//FEATURES
#define F_CTP	"CTP"
#define F_CB	"CB"
#define F_HOG	"HOG"
#define F_SIFT  "SIFT"
#define F_HIST  "HIST"
#define F_LBP	"LBP"

class fm: public FeatureInterface {

	int ID;
	std::string name;
	std::string type;
	CvRect rect;

public:
	fm(int _ID, std::string _name, std::string _type, CvRect _rect) {
		ID = _ID;
		name = _name;
		rect = _rect;
		type = _type;
	}
	~fm(){
		name.clear();
		type.clear();
	}

	int getID(){
		return ID;
	}
	std::string getName() {
		return name;
	}
	std::string getType() {
		return type;
	}
	CvRect getGeometry() {
		return rect;
	}
	lti::matrix<float> getValue() {
		_ERROR("getValue() not implemented !!!");
		exit(-1);
	}
	float getDistance(FeatureInterface* feature) {
		_ERROR("getDistance() not implemented !!!");
		exit(-1);
	}
	void update(FeatureInterface* feature) {
		_ERROR("update() not implemented !!!");
		exit(-1);
	}
	float getWeight() {
		_ERROR("getWeight() not implemented !!!");
		exit(-1);
	}
	void* save() {
		_ERROR("save() not implemented !!!");
		exit(-1);
	}
	void load(void*) {
		_ERROR("load() not implemented !!!");
		exit(-1);
	}

};

class SignatureModel {

public:
	SignatureModel() {
		width = height = 0;
		name = "NULL";
	}

	~SignatureModel(){

		for(uint i = 0; i < features.size(); i++){
			delete features[i];
		}
	}

	SignatureModel(const SignatureModel& m){
		features = m.features;
		name = m.name;
		width = m.width;
		height = m.height;
	}

	std::vector<FeatureInterface*> features;
	std::string name;
	int width;
	int height;

	inline void build(std::string featureName, std::string featureType, CvRect rect, CvSize pattern, CvSize step);
	inline void save(std::string fileName="NULL");
	inline bool load(std::string fileName);
};

void SignatureModel::save(std::string filename) {

	if(filename == "NULL")
		filename = name;

	QDomDocument doc("SIGNATURE_MODEL");

	QDomElement root = doc.createElement("MODEL");
	root.setAttribute("width", width);
	root.setAttribute("height", height);
	root.setAttribute("name", QString(name.c_str()));

	std::vector<FeatureInterface*>::iterator it;

	for (it = features.begin(); it != features.end(); it++) {

		QDomElement feature = doc.createElement("Feature");

		feature.setAttribute("ID", (*it)->getID());
		feature.setAttribute("name", QString((*it)->getName().c_str()));
		feature.setAttribute("type", QString((*it)->getType().c_str()));

		CvRect rect = (*it)->getGeometry();
		feature.setAttribute("rect.x", rect.x);
		feature.setAttribute("rect.y", rect.y);
		feature.setAttribute("rect.width", rect.width);
		feature.setAttribute("rect.height", rect.height);
		root.appendChild(feature);
	}

	doc.appendChild(root);

	QFile file((filename + ".xml").c_str());
	if (!file.open(QIODevice::WriteOnly)) {
		return;
	}

	QTextStream ts(&file);
	ts << doc.toString();
	file.close();
}

bool SignatureModel::load(std::string filename) {

	QFile file((filename + ".xml").c_str());
	if (!file.open(QIODevice::ReadOnly)) {

		_ERROR("file.open( " + filename + " ) crashed!");
		return false;
	}

	QDomDocument doc;
	if (!doc.setContent(&file)) {

		_ERROR("doc.setContent( " + filename + " ) crashed!");
		file.close();
		return false;
	}

	QDomElement root = doc.documentElement();
	if (root.tagName() != "MODEL") {
		_ERROR("XML file: " + filename + ".xml" + " is not a MODEL file!");
		file.close();
		return false;
	}

	name = root.attribute("name", "-1").toStdString();
	width = root.attribute("width", "-1").toInt();
	height = root.attribute("height", "-1").toInt();

	QDomElement feature = root.firstChildElement();
	while (!feature.isNull()) {

		if (feature.tagName() == "Feature") {

			CvRect rect;
			rect.x = feature.attribute("rect.x", "-1").toInt();
			rect.y = feature.attribute("rect.y", "-1").toInt();
			rect.width = feature.attribute("rect.width", "-1").toInt();
			rect.height = feature.attribute("rect.height", "-1").toInt();

			int id = feature.attribute("ID","-1").toInt();
			std::string name = feature.attribute("name", "-1").toStdString();
			std::string type = feature.attribute("type", "-1").toStdString();

			FeatureInterface* f = new fm(id, name, type, rect);
			features.push_back(f);
		}
		feature = feature.nextSiblingElement();
	}

	file.close();

	return true;
}

void SignatureModel::build(std::string featureName, std::string featureType, CvRect rect, CvSize pattern, CvSize step) {

	_MSG("Model:build()");

	width = rect.width;
	height = rect.height;

	std::vector<CvRect> patterns = getPatterns(cvRect(0, 0, width, height), pattern, step);
	std::vector<CvRect>::iterator it;
	int id = 1;
	for (it = patterns.begin(); it != patterns.end(); it++) {
		FeatureInterface* f = new fm(id++,featureName, featureType, *it);
		features.push_back(f);
	}

	name = featureName + IntToString(patterns.size()) + "_" + featureType;

}

#endif /* SIGNATUREMODEL_H_ */
