/*
 * CSEngine.h
 *
 *  Created on: 22 avr. 2010
 *      Author: bboulay
 */

#ifndef CSENGINE_H
#define CSENGINE_H

#include "CSAllTypes.h"
#include "CSAgent.h"
#include "CSAllScenarios.h"
#include "CSScenarioInstance.h"
//#include "CSBasicOperator.h"
//#include "CSAllOperators.h"
#include <sstream>
#include <map>
#include <list>
#include <vector>
#include "CSOperatorInterfaceAbstract.h"
#include <QDir>
#include <QPluginLoader>



class CSEngine {
public:
	CSEngine();
	virtual ~CSEngine();

	//void setAgents(std::map <std::string, std::map<int, std::tr1::shared_ptr<CSAgent> > > i_allAgents);
	void setAllScenarios(CSAllScenarios* i_scenarios);
	void setAllObjectTypes(std::map<std::string, std::vector<std::string> > i_allObjectTypes);
	void setCurrentTime(std::tr1::shared_ptr<CSTimeStamp> i_timeStamp);
	void setCurrentTime(std::tr1::shared_ptr<CSTimeStamp> i_timeStamp, int i_pos);

	void setConstTimeSearch(double i_value);

	void setAllOperators();

	void setHistorySize(int i_history);

	void setThresholdByEventType(std::string i_type, double i_threshold);
	void setLevelAlarm(std::string i_alarm, int i_level);


	//void useOperator(std::string i_name, CSBasicOperator* i_operator);


	//void addAgent(std::tr1::shared_ptr<CSAgent> i_agent);
	//return the id of the agent in the engine
	int getAgent(int i_id, bool i_const = false);
	int getComponent(int i_id);

	//set the attribute value of agent with engine id l_id
	void setAgentAttribute(int i_id, std::string i_attribute, std::tr1::shared_ptr<CSBasicType> i_value, int i_pos);
	void setAgentType(int i_id, std::string i_attribute, std::tr1::shared_ptr<CSString> i_value, int i_pos);
	void setAgentFrameID(int i_id, int i_value);

	//return all the ID in engine of agent with type i_type
	void getAllAgent(std::string i_type, std::vector<int>& io_agents);

	bool isConfident(std::tr1::shared_ptr<CSScenarioInstance>& io_scenario);
	int addRecognisedEvent(std::tr1::shared_ptr<CSScenarioInstance> i_scenario, std::tr1::shared_ptr<CSTimeStamp>& i_begin);
	void removeAgents();
	void removeConstAgents();
	void cleanHistory();
	void cleanRecognisedScenario();


	void readAllObjects(std::string i_file);

	void read(std::string i_file);
	void readEngine(std::string i_file);
	void readEnvVars(std::string i_file);

	void readOperatorAgent(std::string i_file);


	void chooseScenarioModels();
	void checkScenarioModels();
	void checkScenarioModelsConstraints();
	void chooseConstantObjects();
	void optimiseScenariomodels();

	void printRecognisedScenario(std::string i_type = "");
	void printAllRecognisedScenario();

	//create teh possible cases and try to recognise
	void recognisePrimitiveState(std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > >& io_scenariosComplexToRecognise);
	//create the complex cases
	void createComplexEvent(std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > >& io_scenariosToRecognise,
			std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > > i_scenariosComplexToRecognise);
	//recognise the complex cases
	void recogniseComplexEvent(std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > > i_scenariosToRecognise,
			std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > >& io_scenariosComplexToRecognise);
	void infer();



public:

	std::map<std::string, std::vector<std::string> > m_allObjectTypes;
	CSAllScenarios* m_allScenarios;

	std::vector<std::string> m_usedModels;
	std::vector<std::string> m_interestedObjects;

	//std::map <std::string, std::vector<CSAgent*> > m_allAgents;
	//std::map <std::string, std::map<int, std::tr1::shared_ptr<CSAgent> > > m_allAgents; //[Type][ID] = adress
	
	//association between Engine ID of CSAgent instance and a Pointer to the Agent Instance
	std::map<int, std::tr1::shared_ptr<CSAgent> >  m_allAgents; //ID interne;
	std::map<int , CSAgent*> m_tempAgent;


	std::map<int, int> m_equivalentRealID_EngineID;
	std::map<int, int> m_equivalentRealID_EngineIDConst;
	std::map<int, std::tr1::shared_ptr<CSAgent> >  m_allComponents; //Id recognised scenario / instance
	int m_id_counter;


	std::list<std::tr1::shared_ptr<CSScenarioInstance> > m_recognisedScenarios;
	
	int m_number;

	//std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > > m_scenariosToRecognise;
	//std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > > m_scenariosComplexToRecognise;
	std::map<std::string, std::vector<std::tr1::shared_ptr<CSScenarioInstance> > > m_scenarioToRecogniseHistory; //the first complex scenario to recognise

	std::map<int,int> m_removedScenarioID;

	//std::vector<CSScenario*> m_complexScenariosToRecognise;

	//name of operator, operator itself
	//std::map<std::string, CSBasicOperator*> m_allOperators;
	std::map<std::string, CSOperatorInterfaceAbstract*> m_allPluginOperators;
	std::string m_pluginPath;

	std::tr1::shared_ptr<CSTimeStamp> m_currentTime;
	static std::map<int, std::tr1::shared_ptr<CSTimeStamp> > h_currentTime;
	static unsigned int m_historySize;

	//parameters
	double m_constProlongation;
	double m_const_TimeSearch;
	int m_printNumberInstance;
	int m_printNumberRecognised;
	int m_printNumberCheck;

	std::map<std::string, double> 	m_thresholdByEventType; //event type - threshold
	std::map<std::string, int> 	m_levelAlarm; //alarm - level
	double m_bayesianRatio;
	bool m_debug;


};

int permutation(int i_total, int i_desired);
//indice, list permutation
std::vector<std::vector<int > > createPermutationList(std::vector<int > i_values, int i_desired);
bool compare_scenario_end(std::tr1::shared_ptr<CSScenarioInstance> i_scenario1, std::tr1::shared_ptr<CSScenarioInstance> i_scenario2);

std::vector< std::vector<int> > createPermutationList(std::vector<int> i_values);



#endif /* CSENGINE_H_ */
