/**
 * @file	parallelepiped.h
 * @brief	parallelepiped type.
 * @author ORION team (orion@sophia.inria.fr)
 */
 
#ifndef _PARALLELEPIPED_H_
#define _PARALLELEPIPED_H_

#include <set>
#include "geometric.h"

namespace vsip {

struct parallelepiped
{
  struct fpoint_2d base_point[4]; // Four points forming the base of parallelepiped.
  struct fpoint_2d _2D_proj_0[4]; // 2D projections of points in the base
  struct fpoint_2d _2D_proj_h[4]; // 2D projections of points in the base
  double h;         // Height of parallelepiped.
};

typedef struct parallelepiped *parallelepiped_t;

#define PARALL_H(_parall)	((_parall)->h)
#define PARALL_X_i(_parall, _i)	((_parall)->base_point[_i].x)
#define PARALL_Y_i(_parall, _i)	((_parall)->base_point[_i].y)
#define PARALL_POINT_i(_parall, _i)  &((_parall)->base_point[_i])

#define PARALL_X2D_BASE_i(_parall, _i)	((_parall)->_2D_proj_0[_i].x)
#define PARALL_Y2D_BASE_i(_parall, _i)	((_parall)->_2D_proj_0[_i].y)
#define PARALL_2D_BASE_POINT_i(_parall, _i)  &((_parall)->_2D_proj_0[_i])

#define PARALL_X2D_H_i(_parall, _i)	((_parall)->_2D_proj_h[_i].x)
#define PARALL_Y2D_H_i(_parall, _i)	((_parall)->_2D_proj_h[_i].y)
#define PARALL_2D_H_POINT_i(_parall, _i)  &((_parall)->_2D_proj_h[_i])


 parallelepiped_t new_parallelepiped();

 void free_parallelepiped(parallelepiped_t parall);

 parallelepiped_t copy_parallelepiped(parallelepiped_t parall);

 void init_parall_limits(int limits[4][4], int nlimits[4], int varlimrel[4]);

 void get_parall_limits(parallelepiped_t parall, int limits[4][4], int nlimits[4], int varlimrel[4]);
 
 void get_w_l(parallelepiped_t parall, double *w, double *l);

} // end namespace vsip
					 
#endif /* _PARALLELEPIPED_H_ */  
